@@ -509,7 +509,7 @@ class ModelBuilder():
                 
                 ribtimes = np.zeros((400),dtype=np.float64)
                 coltimes = np.zeros((400),dtype=np.int32)
-                ssa_translation_generic.run_SSA_generic(result,ribtimes,coltimes, self.k,frapresult,t_array, np.array([0,0,0],dtype=np.float64), seeds[i],nribs, k_add.flatten()  ,n_enters,n_pauses,n_stops,n_jumps)
+                ssa_translation_lowmem.run_SSA_generic(result,ribtimes,coltimes, self.k,frapresult,t_array, np.array([0,0,0],dtype=np.float64), seeds[i],nribs, k_add.flatten()  ,n_enters,n_pauses,n_stops,n_jumps)
                 all_results[i,:] = result
                 all_frapresults[i,:] = frapresult
                 all_coltimes[i,:] = coltimes
@@ -528,7 +528,7 @@ class ModelBuilder():
                 
                 
 
-                ssa_translation_generic_lowmem.run_SSA_generic(result,kelong ,frapresult,t_array, np.array([0,0,0],dtype=np.float64), seeds[i], k_add.flatten()  ,n_enters,n_pauses,n_stops,n_jumps, probe, ncolors ,max_ribs )
+                ssa_translation_lowmem.run_SSA_generic_lowmem(result,kelong ,frapresult,t_array, np.array([0,0,0],dtype=np.float64), seeds[i], k_add.flatten()  ,n_enters,n_pauses,n_stops,n_jumps, probe, ncolors ,max_ribs )
                 all_results[i,:,:] = result
                 #all_frapresults[i,:] = frapresult
         
